@@ -16,9 +16,12 @@ GNU General Public License for more details.
 For detail about GNU see <http://www.gnu.org/licenses/>.
 '''
 
-import latlon
 from typing import Tuple 
 import math
+import latlon
+
+# from geographiclib.geodesic import Geodesic
+# geod = Geodesic.WGS84
 
 EARTH_RADIUS = 60.0 * 360 / (2 * math.pi) # nm
 NAUTICAL_MILE_IN_KM = 1.852
@@ -37,15 +40,19 @@ def ortodromic2 (lat1: float, lon1: float, lat2: float, lon2: float) -> Tuple[fl
 	return (EARTH_RADIUS * c, a)
 
 def ortodromic (latA: float, lonA: float, latB: float, lonB: float) -> Tuple[float, float]:
+	# g = geod.Inverse(latA, lonA, latB, lonB)
+	# return (g['s12'] * 1e-3, math.radians (g['azi1']))
+
 	p1 = latlon.LatLon(latlon.Latitude(latA), latlon.Longitude(lonA))
 	p2 = latlon.LatLon(latlon.Latitude(latB), latlon.Longitude(lonB))
-
 	return (p1.distance (p2), math.radians (p1.heading_initial(p2)))
 
 def lossodromic (latA: float, lonA: float, latB: float, lonB: float) -> Tuple[float, float]:
+	# g = geod.Inverse(latA, lonA, latB, lonB)
+	# return (g['s12'] * 1e-3, math.radians (g['azi1']))
+	
 	p1 = latlon.LatLon(latlon.Latitude(latA), latlon.Longitude(lonA))
 	p2 = latlon.LatLon(latlon.Latitude(latB), latlon.Longitude(lonB))
-
 	return (p1.distance (p2, ellipse = 'sphere'), math.radians (p1.heading_initial(p2)))
 
 
@@ -61,12 +68,13 @@ def pointDistance (latA: float, lonA: float, latB: float, lonB: float, unit: str
 	p2 = latlon.LatLon(latlon.Latitude(latB), latlon.Longitude(lonB))
 	d = p1.distance (p2)
 
+	# d = ortodromic(latA, lonA, latB, lonB)[0]
+	
 	if unit == 'nm':
 		return km2nm(d)
 	elif unit == 'km':
 		return d
 	
-
 def routagePointDistance (latA: float, lonA: float, distance: float, hdg: float, unit: str='nm') -> Tuple[float, float]:
 	""" Returns the point from (latA, lonA) to the given (distance, hdg) """
 	if unit == 'nm':
@@ -74,6 +82,9 @@ def routagePointDistance (latA: float, lonA: float, distance: float, hdg: float,
 	elif unit == 'km':
 		d = distance
 
+	# g = geod.Direct(latA, lonA, math.degrees(hdg), d * 1e3)
+	# return (g['lat2'], g['lon2'])
+
 	p = latlon.LatLon(latlon.Latitude(latA), latlon.Longitude(lonA))
 	of = p.offset (math.degrees (hdg), d).to_string('D')
 	return (float (of[0]), float (of[1]))
@@ -107,38 +118,31 @@ def reduce180 (alfa: float) -> float:
 		return 0.0
 	return alfa
 
-def pathAsGeojson(path: str):
-	tr = []
-	for wp in path:
-		if len(wp) == 3:
-			tr.append((wp[0], wp[1], str(wp[2]), 0, 0, 0, 0))
-		else:
-			tr.append((wp[0], wp[1], str(wp[4]), wp[5], wp[6], wp[7], wp[8]))
-
+def pathAsGeojson(path) -> object:
 	feats = []
 	route = []
 
-	for order,wayp in enumerate(tr):
+	for order, wayp in enumerate(path):
 		feat = {
 			"type": "Feature",
 			"id": order,
 			"geometry": {
 				"type": "Point",
 				"coordinates": [ # longitude, latitude
-					wayp[1],
-					wayp[0]
+					wayp.pos[1],
+					wayp.pos[0]
 				]
 			},
 			"properties": {
-				"timestamp": str(wayp[2]),
-				"twd": math.degrees(wayp[3]),
-				"tws": wayp[4],
-				"knots": wayp[5],
-				"heading": wayp[6]
+				"timestamp": str(wayp.time),
+				"twd": math.degrees(wayp.twd),
+				"tws": wayp.tws,
+				"knots": wayp.speed,
+				"heading": wayp.brg
 			}
 		}
 		feats.append(feat)
-		route.append([wayp[1], wayp[0]]) # longitude, latitude
+		route.append([wayp.pos[1], wayp.pos[0]]) # longitude, latitude
 
 	feats.append({
 		"type": "Feature",
@@ -148,8 +152,8 @@ def pathAsGeojson(path: str):
 			"coordinates": route
 		},
 		"properties": {
-			"start-timestamp": str(tr[0][2]),
-			"end-timestamp": str(tr[-1][2])
+			"start-timestamp": str(path[0].time),
+			"end-timestamp": str(path[-1].time)
 		}
 	})
 
